/**
 * Copyright (c) 2014-present, Facebook, Inc. All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 */

'use strict';var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {return typeof obj;} : function (obj) {return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;};



var escapeHTML = require('./lib/escapeHTML');






















var HTML_ELEMENT_REGEXP = /(HTML\w*?Element)|Text|Comment/;
var test = isHTMLElement;

function isHTMLElement(value) {
  return (
    value !== undefined &&
    value !== null && (
    value.nodeType === 1 || value.nodeType === 3 || value.nodeType === 8) &&
    value.constructor !== undefined &&
    value.constructor.name !== undefined &&
    HTML_ELEMENT_REGEXP.test(value.constructor.name));

}

function printChildren(flatChildren, print, indent, colors, opts) {
  return flatChildren.
  map(function (node) {
    if ((typeof node === 'undefined' ? 'undefined' : _typeof(node)) === 'object') {
      return print(node, print, indent, colors, opts);
    } else if (typeof node === 'string') {
      return colors.content.open + escapeHTML(node) + colors.content.close;
    } else {
      return print(node);
    }
  }).
  join(opts.edgeSpacing);
}

function printAttributes(attributes, indent, colors, opts) {
  return attributes.
  sort().
  map(function (attribute) {
    return (
      opts.spacing +
      indent(colors.prop.open + attribute.name + colors.prop.close + '=') +
      colors.value.open + ('"' +
      attribute.value + '"') +
      colors.value.close);

  }).
  join('');
}

var print = function print(
element,
_print,
indent,
opts,
colors)
{
  if (element.nodeType === 3) {
    return element.data;
  } else if (element.nodeType === 8) {
    return (
      colors.comment.open + '<!--' + element.data + '-->' + colors.comment.close);

  }

  var result = colors.tag.open + '<';
  var elementName = element.tagName.toLowerCase();
  result += elementName + colors.tag.close;

  var hasAttributes = element.attributes && element.attributes.length;
  if (hasAttributes) {
    var _attributes = Array.prototype.slice.call(element.attributes);
    result += printAttributes(_attributes, indent, colors, opts);
  }

  var flatChildren = Array.prototype.slice.call(element.childNodes);
  if (!flatChildren.length && element.textContent) {
    flatChildren.push(element.textContent);
  }

  var closeInNewLine = hasAttributes && !opts.min;
  if (flatChildren.length) {
    var children = printChildren(flatChildren, _print, indent, colors, opts);
    result +=
    colors.tag.open + (
    closeInNewLine ? '\n' : '') +
    '>' +
    colors.tag.close +
    opts.edgeSpacing +
    indent(children) +
    opts.edgeSpacing +
    colors.tag.open +
    '</' +
    elementName +
    '>' +
    colors.tag.close;
  } else {
    result +=
    colors.tag.open + (closeInNewLine ? '\n' : ' ') + '/>' + colors.tag.close;
  }

  return result;
};

module.exports = { print: print, test: test };